#include "Rahmen.h"
#include "Punkt.h"
#include "Bild.h"
#include "Scroll.h"
#include "ToolTip.h"
#include "Text.h"

using namespace Framework;

// Inhalt der Rahmen Klasse aus Rahmen.h
// Konstruktor 
Rahmen::Rahmen()
    : Zeichnung(),
    br( 1 ),
    farbe( 0xFF000000 ),
    alpha( 0 )
{}

// Destruktor
Rahmen::~Rahmen()
{}

// nicht constant 
void Rahmen::setRamenBreite( int br ) // setzt die Breite des Rahmens
{
    this->br = br;
    rend = 1;
}

void Rahmen::setAlpha( bool a ) // Legt fest, ob der Alphawert der Farbe bercksichtigt werden soll
{
    alpha = a;
    rend = 1;
}

void Rahmen::setFarbe( int f ) // Legt die Farbe des Rahmens fest
{
    farbe = f;
    rend = 1;
}

int Rahmen::getFarbe() const // Gibt die Farbe des Ramens zurck
{
    return farbe;
}

bool Rahmen::hatAlpha() const // Gibt zurck, ob der Alphawert der Farbe beachtet wird
{
    return alpha;
}

int Rahmen::getRBreite() const // Gibt die Breite des Rahmens zurck
{
    return br;
}


// Inhalt der LRahmen Klasse aus Rahmen.h
// Konstruktor 
LRahmen::LRahmen()
    : Rahmen()
{}

// Destruktor
LRahmen::~LRahmen()
{}

void LRahmen::render( Bild &Obj ) // Zeichnet den Rahmen in das RenderZeichnung
{
	Zeichnung::render( Obj );
    int x = pos.x;
    int y = pos.y;
    int b = x + gr.x - 1;
    int h = y + gr.y - 1;
    if( alpha )
    {
        for( int i = 0; i < br; ++i )
        {
            Obj.drawLinieHAlpha( x + i + 1, y + i, gr.x - i * 2 - 1, farbe );
            Obj.drawLinieVAlpha( b - i, y + i + 1, gr.y - i * 2 - 2, farbe );
            Obj.drawLinieHAlpha( x + i + 1, h - i, gr.x - i * 2 - 1, farbe );
            Obj.drawLinieVAlpha( x + i, y + i, gr.y - i * 2, farbe );
        }
    }
    else
    {
        for( int i = 0; i < br; ++i )
        {
            Obj.drawLinieH( x + i + 1, y + i, gr.x - i * 2 - 1, farbe );
            Obj.drawLinieV( b - i, y + i + 1, gr.y - i * 2 - 2, farbe );
            Obj.drawLinieH( x + i + 1, h - i, gr.x - i * 2 - 1, farbe );
            Obj.drawLinieV( x + i, y + i, gr.y - i * 2, farbe );
        }
    }
}

Zeichnung *LRahmen::dublizieren() const // Kopiert das Zeichnung
{
    Rahmen *obj = new LRahmen();
    obj->setPosition( pos );
    obj->setSize( gr );
    obj->setMausEreignisParameter( makParam );
    obj->setTastaturEreignisParameter( takParam );
    obj->setMausEreignis( mak );
    obj->setTastaturEreignis( tak );
    if( toolTip )
        obj->setToolTipText( toolTip->zText()->getText(), toolTip->zBildschirm() );
    obj->setAlpha( alpha );
    obj->setFarbe( farbe );
    obj->setRamenBreite( br );
    return obj;
}


// Inhalt der Rahmen3D Klasse aus Rahmen.h
// Konstruktor 
Rahmen3D::Rahmen3D()
    : Rahmen()
{
    alpha = 1;
    farbe = 0x70FFFFFF;
    br = 5;
}

// Destruktor
Rahmen3D::~Rahmen3D()
{}

void Rahmen3D::render( Bild &Obj ) // Zeichnet den Rahmen in das RenderZeichnung
{
    Zeichnung::render( Obj );
    int x = pos.x;
    int y = pos.y;
    int b = x + gr.x - 1;
    int h = y + gr.y - 1;
    int fcomp = ( farbe & 0xFF000000 ) | ( ~farbe & 0x00FFFFFF );
    if( alpha )
    {
        for( int i = 0; i < br; ++i )
        {
            Obj.drawLinieHAlpha( x + i + 1, y + i, gr.x - i * 2 - 1, farbe );
            Obj.drawLinieVAlpha( b - i, y + i + 1, gr.y - i * 2 - 2, farbe );
            Obj.drawLinieHAlpha( x + i + 1, h - i, gr.x - i * 2 - 1, fcomp );
            Obj.drawLinieVAlpha( x + i, y + i, gr.y - i * 2, fcomp );
        }
    }
    else
    {
        for( int i = 0; i < br; ++i )
        {
            Obj.drawLinieH( x + i + 1, y + i, gr.x - i * 2 - 1, farbe );
            Obj.drawLinieV( b - i, y + i + 1, gr.y - i * 2 - 2, farbe );
            Obj.drawLinieH( x + i + 1, h - i, gr.x - i * 2 - 1, fcomp );
            Obj.drawLinieV( x + i, y + i, gr.y - i * 2, fcomp );
        }
    }
}

Zeichnung *Rahmen3D::dublizieren() const // Kopiert das Zeichnung
{
    Rahmen *obj = new LRahmen();
    obj->setPosition( pos );
    obj->setSize( gr );
    obj->setMausEreignisParameter( makParam );
    obj->setTastaturEreignisParameter( takParam );
    obj->setMausEreignis( mak );
    obj->setTastaturEreignis( tak );
    if( toolTip )
        obj->setToolTipText( toolTip->zText()->getText(), toolTip->zBildschirm() );
    obj->setAlpha( alpha );
    obj->setFarbe( farbe );
    obj->setRamenBreite( br );
    return obj;
}